# 14.08.2016

# T Aykanat, M Lindqvist, V L Pritchard, and C R Primmer (2016) "From population genomics to conservation and management: a workflow for targeted analysis of markers identified using genome-wide approaches in Atlantic salmon (Salmo salar)."
# e-mail: Tutku Aykanat  tutku.aykanat@utu.fi.

setwd("FILE PATH") ### 

# MAIN CODE ##

# This is code helps user to call genotypes from raw data.

# MODULE 1: Preliminary files/functions/libraries that are require to upload.
{
     #libraries
     library (Biostrings) #This R package contains functions for doing modifications in teh DNA sequences.
     library(snow) # parallel computing
     
     #objects
     lowQ = strsplit(as.character(PhredQuality(0:18)),"")[[1]] # This is the object which defines the lowQ thershold level.
     load(file="SET_FINAL") # This is an important tabular object which provides primers, product sequence, SNP variant, and other relevant infromation of loci in the panels. This object is used to match raw reads to target loci and to call the SNP variant (genotyping). A target locus is matched using forward and reverse primer infromation, as well as the recognition sequence, which is the seqeunce information around the SNP site. Note that, for a few SNPs, the way the recognition sequence selected deviates from from the description in the MS, and for very few cases an  adapter seqeunces has been included to further improve genotype topogrphy (e.g. figure 5 in the paper). Finally, the total number of SNPs in this object is 197. About 20 SNPs from Aykanat et al 2016 (10% of 216 SNP) is not included here. Our post-acceptance results using a larger number of individuals suggests genotype calls of some SNPs exhibited some disconcordance between 7K data and ion-torrent geneotypigng. Therfore, we do not advice to include thsose SNPs in the anlaysis. SNPs provided here had >99% conconrdance with 7K SNP data across 192 individuals (unpublished data). Likeiwise, SNPs taken from 220K SNPS (i.e. sea age SNPs) has high (>99%) conconrdance. Finally, this diagnostoc test is optimised for Teno river Atlantc salmon and slight changes in genotype calling topography are likley when used with phenogentcially distant lineages. We advice an intial visual inspection of coverage topogrpagy.  
     
     revtag ="CATTAAGTTCCCATTA" # this is reverse tag at the end of reverse primer. This tag is targeted in PCR-2 as reverse site. 
     fwdtag ="ACGACGTTGTAAAA" # this is forward tag at the end of reverse primer. This tag is targeted in PCR-2 as forward site. Any full length product seqeunced should have both seqeunce in the fastq file.

     # preparing barcode sequences to use in matching 
     revbarc_i01to08 = c ( "CGTGATCATTAAGT","ACATCGCATTAAGT","GCCTAACATTAAGT","TGGTCACATTAAGT","CACTGTCATTAAGT","ATTGGCCATTAAGT","GATCTGCATTAAGT","TCAAGTCATTAAGT") 
     revbarc_i01to08_comp = sapply(1:length(revbarc_i01to08), function(i) { as.character(reverseComplement( DNAString(revbarc_i01to08[i] ))) })
     revbarc_i01to08_names = paste("Ion-TrPi-i0",1:8,sep="")

     # the file with sample names, and the reverse and forward barcode for each sample. 
     samples = read.table("XXX.txt",h=T) # sample name file with forward and reverse barcode information (sample name 1st column, left barcode and right barcode in 3rd and 4th column)
     # left barcode has "Ion-A_IonX_" name in front of the number (e.g. "Ion-A_IonX_001", "Ion-A_IonX_002",... etc), and right barcode has "Ion-TrPi-i" in front of numbers (e.g. "Ion-TrPi-i03", "Ion-TrPi-i04", .. etc.)
     
     # These are files from iontorrent output. In our case individuals are seperated by their "front tag" automatically. These files are only diffrenet in their index number, so I only change index "i" to call them one by one. normally, an R object with file names will work just as good. If teh seqeucning platform does not seperate samples based on fwd barcode, one should slightly change this code. 
     #"C:/Users/tutayk/Desktop/PS/PS ION TORRENT RUN/TSP_SAMPLE_OPT_BARCODE_H1/IonXpress_001_TSP_SAMPLE_OPT_BARCODE.fastq" an example file with full path
     fileIndex = sapply(1:96, function(i) { ifelse( (nchar(i)==2) , paste("0",i,sep="") , { paste("00",i,sep="") } ) })
     fastqFile = paste("C:/Users/tutayk/Desktop/PS/PS ION TORRENT RUN/TSP_SAMPLE_OPT_BARCODE_H2/IonXpress_", fileIndex,"_TSP_SAMPLE_OPT_BARCODE.fastq",sep="")
     fastqFile ## a vector to call files with seperate barcodes.
     
     
}     
# MODULE 1 ENDS.


# MODULE 2: uploading FASTQ files.
{
     listSeq3 = list()
     listPhred3 = list()
     
     for ( i in 1:nrow(samples) ) { print(i)
             
          identifierFWD = gsub("Ion-A_IonX_","",samples[i,3])
          identifierREV = samples[i,4]
          file = fastqFile [grep(identifierFWD,fastqFile)]
          FastQ.1 = read.table(file,stringsAsFactors=F,comment.char = "") # indiviual fastq files for each q
          FastQ.seq = FastQ.1 [ seq(2,nrow(FastQ.1),by=4),1] # extract only only sequence from the fastq file. Note that one in every four lines are sequence, one phred quality scores, and the other two are identifiers. We dont need to have identifiers in te belwo code. 
          FastQ.phred  = PhredQuality(FastQ.1 [ seq(4,nrow(FastQ.1),by=4),1]) # extract only only phred quality score from the fastq file.		
          REVBARC = revbarc_i01to08_comp [which(identifierREV==revbarc_i01to08_names)]
          ind1 = which ( vcountPattern (REVBARC , FastQ.seq , max.mismatch = 1, with.indels=T, fixed=T) == 1 ) # this is match to reverse barcode #2    
          listSeq3 [[ i ]] = FastQ.seq  [ind1]
          listPhred3 [[ i ]] = FastQ.phred  [ind1]
     }
     
     names(listSeq3) = samples[,1]
     names(listPhred3) = samples[,1]
     
     save(file="listSeq3_H2",listSeq3)
     save(file="listPhred3_H2",listPhred3)

}
# MODULE 2 ENDS.


# MODULE 2 alternative: uploading FASTQ files (this code ignores sample information, it uses combinations of forward and reverse barcodes as indicdtaed by user).
{
     
     
          # 

     	listSeq = list() # we will staore sequence information here
     	listPhred = list() # we will store quality information here
     	
          # below is a loop to fill the above list  
     	for (k in 1:length(fastqFile)) {  print(k)	     
     		indFastQall = read.table(fastqFile[k],stringsAsFactors=F,comment.char = "") # indiviual fastq files for each q
     		indSeq = indFastQall [ seq(2,nrow(indFastQall),by=4),1] # extract only only sequence from the fastq file. Note that one in every four lines are sequence, one phred quality scores, and the other two are identifiers. We dont need to have identifiers in te belwo code. 
     		indPhred = PhredQuality(indFastQall [ seq(4,nrow(indFastQall),by=4),1]) # extract only only phred quality score from the fastq file.		
     		listSeq[[k]] =  indSeq
     		listPhred[[k]] =  indPhred 
          }
     
          ## sorting individuals using 6 bp reverse barcodes.Additional 8 bp is added from the "reverse tag" (PCR-2) to minimise non-spcifiafc match (CATTAAGT). Note taht if you dont have a reverse barcode you can skip this stage. One might have more than two reverse barcodes, diffenet sequences etc.. Code should be modified accprdingly. 

                         
          # listSeq2 = listSeq # USE THIS and skip rest of teh module if you did not specify a reverse barcode.
          # listPhred2 = listPhred # USE THIS and skip rest of teh module if you did not specify a reverse barcode.
          
          # list object to store individuals sequences
          listSeq2 = list()
          listPhred2 = list()
     
          # this is the loop to sort out individuals by reverse barcode. NOte that e use "vcountPattern" function from Biostring pacakge, and allowing 1 mismatch.
          # in this particular case we have revbarc_i01to08 [c(2,6,7)] as the reverse barcodes
          REVBARC = revbarc_i01to08_comp [c(2,6,7)]
          
               for (i in 1:length(listSeq)) {  print(i)

                    #in this particular case we have revbarc_i01to08 [c(2,6,7)] as the reverse barcodes
               	for (j in 1:length(REVBARC)) {
               	 
               	ind1 = which ( vcountPattern (REVBARC[j] , listSeq[[i]] , max.mismatch = 1, with.indels=T, fixed=T) == 1 ) # this is match to reverse barcode #2    
               	listSeq2[[ (length(listSeq) * (j-1) ) + i ]] = listSeq [[i]]  [ind1]
               	listPhred2[[ (length(listSeq) * (j-1) ) + i ]] = listPhred [[i]]  [ind1]
     
               }
          }  
          
          
          #list object to store sequences that barcode information was not found (not important but for recorind purposes)
          numreads_fwd = sum(as.numeric(summary(listSeq)[,1])) ## this is total number of reads with a forward barcoded.
          numreads_fwdrev = sum(as.numeric(summary(listSeq2)[,1]),na.rm=T) ## this is total number of reads with a reverse barcode qithin fwd barcodes.
          numreads_fwdrev / numreads_fwd
          
}     
# MODULE 2 alternative ENDS.


     #MODULE 2 OUTPUT
     #listSeq3: raw sequences (reads) listed by individual's index.
     #listPhred3: corresponding phred scores.

#~~~ MODULE 3: Calling nucleotide bases on the SNP position  ~~~#
{

     # This module is the major workhorse for calling the genotypes. 1) Each targeted locus is called and quaility is assesed. 
     # This module uses raw sequence as input. "SET_FINAL" file is used to match primer, and SNP site sequences in the raw seqeunces.
     # this function is slow (> 2-3 hours but less than a day), so we use the R pacake "snow" which allows parallel computing. 
     cl <- makeSOCKcluster(rep("localhost",5)) # I have eight cores in my work computer, and I allocated four-six of them for this job.
     
     # objects, libraries used in parallel compueting  needs to be intruduced to each parallel nodes, by the below function.
          clusterExport(cl, "listSeq3") 
          clusterExport(cl, "listPhred3")
          clusterExport(cl, "SET_FINAL")
          clusterExport(cl, "lowQ")
          clusterEvalQ(cl, library("Biostrings"))
     

          COVSpre = parSapply(cl, 1:length(listSeq3) ,  function(j) { print(j)      # function is run for each individual (e.g. from 1 to length(listSeq3) )
               sapply(1:length(SET_FINAL$FWD)  , function(i) { print(i) # for each individual each loci run seperatlt. Therfore, the belwo code is individual AND locus specific
          		               
          			SEQ1 = SET_FINAL [i,6]  # the expected seqeunce including primers
          			FWD1 = SET_FINAL [i,4]  # forward primer sequence
          			REV1 = SET_FINAL [i,7]  # reverse primer seqeunce
          			Nind = SET_FINAL [i,12] # the recognition seqeunce with SNP seqeunce marked as N.
          			reverseComplementIndex = SET_FINAL[i,10]==SET_FINAL[i,11] # this is a compatibality boolean for identify the SNP consistently with 7K illumina chip calling. Ask Tutku for details. 
          			Nind2 = Nind # back compatiablity 
          			Nind3 = SET_FINAL[i,13] # adapter sequence. this and  te next line defines seqeunce to identfy tru copy from the duplicates. In some cases the duplicate loci has identical fwd and rev primers and recognition seqeunce, so we take a sequence in the amplicon with fixed differences between duplicates to only select the loci of interest.
          			Nind4 = SET_FINAL[i,14] # anti adapter
          		
               	# the below function is to estimate coverages. note tat ifelse is only for SDY (sex) loci.
               	covs2 = if (!is.na(Nind2)) {
               								
               			RC = vcountPattern ( REV1, listSeq3 [[j]] , max.mismatch = 1, with.indels=T, fixed=T ) # indices of seqeunces to reverse primer match (1 mismatch allowed).
               			FC = vcountPattern ( FWD1, listPhred3 [[j]] , max.mismatch = 1, with.indels=T, fixed=T ) # indices of seqeunces to forward primer match (1 mismatch allowed).
                              # note that one can do the above routine by matching sequences to a truncated/shorter primer seqeunce in the intesrtt of increasig coverage and in the expense of specificity. Note that iontorrent specificity is low esp. when polyN regions are present, therfore shorter recognition seqeunces are better.
                         
               				CANDS = listSeq3[[j]] [which( (RC+FC) > 0)]  # indices of candidate sequences (at least one of rev or fwd primer matches to a seqeunce) 
               				CANDSphred = listPhred3[[j]] [which( (RC+FC) > 0)] # same as above but in teh 
               			
                                   CANDS2 = CANDS
                                   CANDSphred2 = CANDSphred
                         
               				if (!is.na(Nind3)) { # ifelse condition to accomodate "adapter" sequence
               					CANDS2 = CANDS [grep ( Nind3, CANDS)]
               					CANDSphred2 =  CANDSphred [grep ( Nind3, CANDS)]
               					}
               
               				if (!is.na(Nind4)) { # ifelse condition to accomodate "antiadapter" sequence
               					CANDS2 = CANDS [!grepl ( Nind4, CANDS)]
               					CANDSphred2 =  CANDSphred [!grepl ( Nind4, CANDS)]
               					}
               				
                              # below four lines searches for A,T,G,C in the SNP region. 
               				AAscanA = regexpr(gsub("N",ifelse(reverseComplementIndex,"A","T"),Nind2), CANDS2) 
               				TTscanA = regexpr(gsub("N",ifelse(reverseComplementIndex,"T","A"),Nind2), CANDS2)
               				GGscanA = regexpr(gsub("N",ifelse(reverseComplementIndex,"G","C"),Nind2), CANDS2)
               				CCscanA = regexpr(gsub("N",ifelse(reverseComplementIndex,"C","G"),Nind2), CANDS2)
               				
                              # below four lines calculate coverage for for each of the bases in the SNP region
                                   AAscanB = which ( AAscanA != -1)
               				TTscanB = which ( TTscanA != -1)	
               				GGscanB = which ( GGscanA != -1)
               				CCscanB = which ( CCscanA != -1)	
                         
                              # below calculates coverage in the "CANDS" other than the above four "specific" ones. In the output it is represnted with a NUMBER. This is surrogate for unspecific contribution by the primer.
               				ALLscanB = length(CANDS)	-length(c(AAscanB,TTscanB,GGscanB,CCscanB))
               				
                              # below marks each "specific" coverage according to its quality. low quality (<20 phred score) marks as 0,  others as 1.      
               				#MARKING A1	
               				AAs = paste(as.numeric(!(substr(CANDSphred [AAscanB],AAscanA[AAscanB],AAscanA[AAscanB])%in% lowQ)), rep("A" , length(CANDS [AAscanB]) ),sep="_")
               				#MARKING A2	
               				TTs = paste(as.numeric(!(substr(CANDSphred [TTscanB],TTscanA[TTscanB],TTscanA[TTscanB]) %in% lowQ)), rep("T" , length(CANDS [TTscanB]) ),sep="_")
               				#MARKING A1	
               				GGs = paste(as.numeric(!(substr(CANDSphred [GGscanB],GGscanA[GGscanB],GGscanA[GGscanB]) %in% lowQ)), rep("G" , length(CANDS [GGscanB]) ),sep="_")
               				#MARKING A2	
               				CCs = paste(as.numeric(!(substr(CANDSphred [CCscanB],CCscanA[CCscanB],CCscanA[CCscanB]) %in% lowQ)), rep("C" , length(CANDS [CCscanB]) ) ,sep="_")
               	          
               				ALLs = c(AAs,TTs,GGs,CCs,ALLscanB)
               				
               			#below else function is to quantify the SDY coverage (sex loci).	It doesnt have an "N" so it is represented by a number in the output.
                         
               					} else { 			
               					seqCOV  = vcountPattern (SEQ1, listSeq3[[j]] , max.mismatch = ifelse ( grepl("n",SEQ1), length(gregexpr("n",SEQ1,"")[[1]])+3, 3), with.indels=T, fixed=T)
               					covs = listSeq3[[j]] [which(seqCOV==1)]
               					rep(1,length(covs))	}
          						
          			covs2	
     		})
     	})
     
          save(COVSpre,file="COVSpreH1")
     
     stopCluster(cl) # this one stops the parallel computing

}
#~~~ MODULE 3 ENDS. ~~~#

     #OUTPUT of MODULE 3.
          # COVSpre: this is the master coverage information file. This list object has coverage infromation for every loci and individual. 

#~~~ MODULE 3b: Fast general descriptives ~~~# 
{    

     readsVecR4 = unlist(COVSpre) # vector form of the coverage master file
     valid = sum (grepl("1",readsVecR4))
     invalid = sum (grepl("0",readsVecR4)) ##these fail the quality
     
     totCov = sum(as.numeric(summary(listSeq3)[,1])) # total coverage. note that this doesnt include the reads without forward barcode, so the result is n overestimation
     totCov = XXX #total coverage real this number is taken from iontorrent server window
     valid/totCov # XXX % valid ONTARGET
     invalid/totCov # XXX % inalid ONTARGET
}
#~~~ MODULE 2b ENDS ~~~#

#~~~ MODULE 3: COVERAGES ETC... ~~~#
# This module outputs several objects. Some trivial while others are more important for downstream anlayses or assay descriptives/diagnostic. 
# Objects' row/column numbers are equal to number of SNPs/individuals, respectively. 
{

load(file="COVSpreH2")
     ## COVS1: Low quality bases, marked 0 (after phred quality assesemnt) are removed from the dataset
	COVS1 = t(sapply( 1: nrow(COVSpre), function(i) { print(i)
		sapply(1: ncol(COVSpre), function(j) { 
			AA = COVSpre[i,j][[1]]
			AA = AA [ is.na(as.numeric(AA))] #removing NUMBER entry (number of seqeunces that contains either of the primer but lacks the specific recognitions seqeunce).
			AA = if ( sum(is.na(AA))== length(AA) ) NA else {
				table(AA [!(is.na(AA) | grepl( 0, AA ))])  }
			AA = if( length(AA) ==0 ) NA else AA 
			AA = if (is.numeric(COVSpre[i,j][[1]]) ) sum(COVSpre[i,j][[1]]) else AA
			list(AA)
			})
	}))

     ## COVS2 = Number of differnet type of nucleotide bases (e.g. A,T,C, and G) called for specific SNP loci and individual. NA= 0, One base type=1, Two base type= 2, Three base type = 3 etc.. This is to filter out or understand the nature SNP region. Third or forth bases may mean miscall,or duplicate presense, among others)
	COVS2 = t(sapply( 1: nrow(COVSpre), function(i) { print(i)
		sapply(1: ncol(COVSpre), function(j) { 
			if ( sum(!is.na(COVS1[i,j][[1]]))==0 |  length(COVS1[i,j][[1]])==0 )	0 else { length(COVS1[i,j][[1]]) } 
	})
		}) )
	     
	## COV2b = Only expected nucleor=tides in the SNP site are retained (using SET_FINAL).
	COVS2b = t(sapply( 1: nrow(COVSpre), function(i) { print(i)
				sapply(1: ncol(COVSpre), function(j) { 
					AA = COVS1[i,j][[1]]
					AA = gsub("1_","",names(AA))
					if(length(AA)==0) 0 else { COVS1[i,j][[1]] [ AA %in% strsplit(SET_FINAL[i,9],"")[[1]] ] }
				})
	}))
	
	COVS2b[which ( SET_FINAL[,2] == "SDY_ion2"),] = COVS1[which ( SET_FINAL[,2] == "SDY_ion2"),] ## adding SDY coverage maunally
          
	## COVS3 = Coverages per ind/marker after incorrect calls (NUMBERS)  removed.
	COVS3 = t(sapply( 1: nrow(COVSpre), function(i) { print(i)
		sapply(1: ncol(COVSpre), function(j) { 
			sum(COVS1[i,j][[1]])
	})
		}) )

	## COVS3b = Coverages per ind/marker after low quality calls (0_N) AND incorrect calls (NUMBERS)  removed.
	COVS3b = t(sapply( 1: nrow(COVSpre), function(i) { print(i)
		sapply(1: ncol(COVSpre), function(j) { 
			sum(COVS2b[i,j][[1]])
	})
		}) )	
	
	# Below object is a boolean matrix to specify if (loci x individuals) combination is assayed or not. In most cases all individuals are genotyped in all SNPs, so all elements in the matrix are 1.
     covMAT = matrix(1,nrow=nrow(COVSpre),ncol=ncol (COVSpre))
     
     #COVS4 = IMPORTANT OBJECT. It shows coverage per individual and locus, but it does not provide infromation on genotype calls. 
     #all loci are inculded for all samples
     COVS4 = COVS3b * covMAT
	COVS4[which ( SET_FINAL[,2] == "SDY_ion2"),]=  unlist(COVS1[which ( SET_FINAL[,2] == "SDY_ion2"),]) # again adding SDY coverage manually
     
          # again, some descriptives          
          sum(COVS4>10,na.rm=T) / sum(!is.na(COVS4)) # XXX% is higher than 10x coverage
     	sum(COVS4>13,na.rm=T) / sum(!is.na(COVS4)) # XXX% is higher than 13x coverage
     	sum(COVS4>30,na.rm=T) / sum(!is.na(COVS4)) # XXX% is higher than 20x coverage
     
          #coverege per indvidual     
          aveCOV1 = round (sapply ( 1:dim(COVS4)[2], function(i) { sum(unlist(COVS1[,i]),na.rm=T) / dim(COVS4)[1] }) )
          #coverege per locus
          aveCOV2 = round (sapply ( 1:dim(COVS4)[1], function(i) { sum(unlist(COVS1[i,]),na.rm=T) / dim(COVS4)[2] }) )

          
     par(mfrow=c(3,1))
     plot(sort(aveCOV2),xlab="loci sorted by cov",ylab="coverage",pch=20)
     plot(sort(aveCOV1),xlab="individuals sorted by cov",ylab="coverage",pch=20)
     plot(COVS4[6,]/aveCOV1,pch=20,xlab="samples",ylab="normalized SDY coverage",ylim=c(0,4))

     }
#~~~ MODULE 3 ENDS ~~~#

#~~~ MODULE GENOTYPING ~~~#
{
     # Here we adopted campbell et al 2015 method to call for genotypes. See their figure 4 (or 5) for more details.
     # We adopted a "transformation of axis" based methodllogy to account for MVN loci with stringent criteria for calling genotypes.
     
     COVSg1 = COVS2b

     #this matrix counts valid As only.
     COVSg1_A = t(sapply( 1: nrow(COVSg1), function(i) { print(i)
          		 sapply( 1: ncol(COVSg1), function(j)  {
     				A = as.numeric(COVSg1[i,j][[1]]["1_A"] )
     			 	if (is.na(A)) 0 else A
     			 	})	}))
     #this matrix counts valid Ts only.
     COVSg1_T = t(sapply( 1: nrow(COVSg1), function(i) { print(i)
     			 sapply( 1: ncol(COVSg1), function(j)  {
     				A = as.numeric(COVSg1[i,j][[1]]["1_T"] )
     			 	if (is.na(A)) 0 else A
     			 	})	}))
     #this matrix counts valid Gs only.
     COVSg1_G = t(sapply( 1: nrow(COVSg1), function(i) { print(i)
     			 sapply( 1: ncol(COVSg1), function(j)  {
     				A = as.numeric(COVSg1[i,j][[1]]["1_G"] )
     			 	if (is.na(A)) 0 else A
     			 	})	}))
     #this matrix counts valid Cs only.
     COVSg1_C = t(sapply( 1: nrow(COVSg1), function(i) { print(i)
     			 sapply( 1: ncol(COVSg1), function(j)  {
     			 	A = as.numeric(COVSg1[i,j][[1]]["1_C"] )
     			 	if (is.na(A)) 0 else A
     			 	})	}))
     
     COVSg1_A = COVSg1_A * covMAT
     COVSg1_T = COVSg1_T * covMAT
     COVSg1_G = COVSg1_G * covMAT
     COVSg1_C = COVSg1_C * covMAT
     
     AA = apply(COVSg1_A,1, function(x) { mean(x,na.rm=T) } )
     TT = apply(COVSg1_T,1, function(x) { mean(x,na.rm=T) } )
     GG = apply(COVSg1_G,1, function(x) { mean(x,na.rm=T) } )
     CC = apply(COVSg1_C,1, function(x) { mean(x,na.rm=T) } )
     
     #average coverage per loci, per base.  
     round(cbind(AA,TT,GG,CC))

     ### this to avoid infinite numbers when estimating coverages. (based on campbell et al 2004)
     COVSg1_A [COVSg1_A==0]=0.1
     COVSg1_T [COVSg1_T==0]=0.1
     COVSg1_G [COVSg1_G==0]=0.1
     COVSg1_C [COVSg1_C==0]=0.1

     GENCALL1cov =  matrix(NA, nrow(COVSg1),ncol(COVSg1) )
     GENCALL2cov =  matrix(NA, nrow(COVSg1),ncol(COVSg1) )
     GENCALL =  matrix(NA, nrow(COVSg1),ncol(COVSg1) )
     GENCALLnumbers =  matrix(NA, nrow(COVSg1),ncol(COVSg1) )
     
     # Transfromation of MVN is given in SET_FINAL$MVNlike column. If 0 it is not MVN, other values gives tranfromation angle

     # this double loop fills in coverage of specific SNPs per individuals
     # current thrshold setting normal loci: coverege = 10x , pp < 0.1, qq > 10, 0.2 < pq < 5
     # current thrshold setting MVN loci: coverege = 20x , pp < 0.67, qq > 12.5, 0.25 < pq < 4 (much stringent calling criteria for MVN)
     # -9 depits fail due to not passing the therhold. NA depits failiure due to failure the proprtion of allele coverage thrhold

for ( i in 1:nrow(COVSg1) ) { print(i) # this one runs locus by locus
          
     for ( j in 1:ncol(COVSg1) ) {  # this one runs individuals by individuals. SO each locus (index "i") is executed for every individual (insex "j")       
          
          if (SET_FINAL$MVNlike[i] == 0) {
                    
                         alleles = strsplit ( SET_FINAL[i,9] , "" ) [[1]] # selects only targeted bases at the selected loci
     				
                         VAR = which(c("A","T","G","C") %in% alleles) # targeted alleles are selected here
     
                         A1 = c("A","T","G","C") [VAR][1] # tareget allele 1
     				A2 = c("A","T","G","C") [VAR][2] # tareget allele 2
     				
     				COVSggg =  c(COVSg1_A [i,j],COVSg1_T [i,j],COVSg1_G [i,j],COVSg1_C [i,j]) 
     				COVSggg2 = COVSggg [VAR] #only infromation related to targetd alleles are kept
     				
     				VALgg = COVSggg2[1] / COVSggg2[2]
     				
     				VALgg2 = if ( is.na(VALgg) ) NA else {	
     							if(  (COVSggg2[1] + COVSggg2[2])  < 10 ) -9 else { ## loci with less than 10 coverage is not called
     								if( VALgg<0.1 ) paste(A2,A2,sep="") else { ## ratio of coverage <0.1 and >10 are called homozygote for each allele
     									if( VALgg>10 ) paste(A1,A1,sep="") else {
     										if( VALgg>0.2 & VALgg<5) paste(sort(c(A1,A2)),collapse="",sep="") else {NA} }}}} ## coverege proprtion between 0.2. and 5 are called heterzygote.
                    
     				VALgg2numbers = if ( is.na(VALgg) ) NA else {	
     							if(  (COVSggg2[1] + COVSggg2[2])  < 10 ) -9 else {
     								if( VALgg<0.1 ) 0 else {
     									if( VALgg>10 ) 2 else {
     										if( VALgg>0.2 & VALgg<5) 1 else {NA} }}}}

                    GENCALL1cov [i,j] = COVSggg2[1]
     	          GENCALL2cov [i,j] = COVSggg2[2] 
     	          GENCALL [i,j] = VALgg2 
     	          GENCALLnumbers [i,j] = VALgg2numbers 

             } 
               
          if (SET_FINAL$MVNlike[i] > 0)  {
               
     		alleles = strsplit ( SET_FINAL[i,9] , "" )[[1]] # selects only targeted bases at the selected loci
     		
               VAR = which(c("A","T","G","C") %in% alleles) # targeted alleles are selected here
     
               A1 = c("A","T","G","C") [VAR][1] # tareget allele 1
     		A2 = c("A","T","G","C") [VAR][2] # tareget allele 2
     				
     		COVSggg =  c(COVSg1_A [i,j],COVSg1_T [i,j],COVSg1_G [i,j],COVSg1_C [i,j]) 
     		COVSggg2 = COVSggg [VAR] #only infromation related to targetd alleles are kept
     		
     		degree1 = ( 180 / ( pi / atan ( COVSggg2[2] / COVSggg2[1] )  ))
               degree2 = round ( ifelse ( degree1 > 90, 90, ifelse ( degree1 < SET_FINAL$MVNlike[i] , SET_FINAL$MVNlike[i] , degree1 )))
               KK = round ( seq(SET_FINAL$MVNlike[i] ,90,by=1) * seq(1,0,length.out = length(SET_FINAL$MVNlike[i]:90)) )
               seq(SET_FINAL$MVNlike[i] ,90,by=1) 
               
               KK2 = KK [which(degree2 == seq(SET_FINAL$MVNlike[i] ,90,by=1) )]

               degree3 = degree2-KK2
               R = sqrt ( (COVSggg2[2])^2 + (COVSggg2[1])^2 )
               YYY1 = round ( sin ( pi * (degree3/180) ) * R )
               XXX1 = round ( cos ( pi * (degree3/180) ) * R )
               c(XXX1,YYY1)
              
               COVSggg2[1] = XXX1
     		COVSggg2[2] = YYY1
     		
     				VALgg = COVSggg2[1] / COVSggg2[2]

     				VALgg2 = if ( is.na(VALgg) ) NA else {	
     							if(  (COVSggg2[1] + COVSggg2[2])  < 20 ) -9 else { ## loci with less than 20 coverage is not called
     								if( VALgg<0.067 ) paste(A2,A2,sep="") else { ## ratio of coverage <0.067 and >12.5 are called homozygote for each allele
     									if( VALgg>12.5 ) paste(A1,A1,sep="") else {
     										if( VALgg>0.25 & VALgg<4) paste(sort(c(A1,A2)),collapse="",sep="") else {NA} }}}} ## coverege proprtion between 0.25. and 4 are called heterzygote.
                    
     				VALgg2numbers = if ( is.na(VALgg) ) NA else {	
     							if(  (COVSggg2[1] + COVSggg2[2])  < 20 ) -9 else {
     								if( VALgg<0.067 ) 0 else {
     									if( VALgg>12.5 ) 2 else {
     										if( VALgg>0.25 & VALgg<4) 1 else {NA} }}}}
     				
     	GENCALL1cov [i,j] = COVSggg2[1]
     	GENCALL2cov [i,j] = COVSggg2[2] 
     	GENCALL [i,j] = VALgg2 
     	GENCALLnumbers [i,j] = VALgg2numbers 

     				
               }
               
          }
          
     }
     
     GENCALL1cov[1:10,1:10] # coverage allele 1
     GENCALL2cov[1:10,1:10] # coverage allele 2
     GENCALL[1:10,1:10] # genotype call (-9 for no call)
     GENCALLnumbers[1:10,1:10] # number of diffrenet bases in the call

     #(snmall descriptive) call failiure is larger in MVN due to stringent criteria
     normalloci = as.vector(GENCALL [which(SET_FINAL$MVNlike==0),])
     sum(is.na(normalloci)) / length(normalloci)
     MVNloci = as.vector(GENCALL [which(SET_FINAL$MVNlike!=0),])
     sum(is.na(MVNloci)) / length(MVNloci)

     #genotype success rate per individuals
     genoSuccessInd = sapply( (1:ncol(GENCALL)), function(j) {
          AA1 = GENCALL[,j]
     	AA2 = AA1 [!is.na(AA1)]
     	AA3 = 1-sum(grepl("-9",AA2))/length(AA2)
     })


     #genotype success rate per loci

     genoSuccessLoci = sapply( (1:nrow(GENCALL))[ ] , function(i) { #1:nrow(GENCALL))[-8]
     	AA1 = GENCALL[i,] # GENCALL[i,-8]
     	AA2 = AA1 [!is.na(AA1)]
     	AA3 = 1-sum(grepl("-9",AA2))/length(AA2)
     })
     
     genoSuccessLoci [which ( SET_FINAL[,2] == "SDY_ion2")] = 1 ## sdy sexing loci is eqauled to 1

     par(mfrow=c(2,1))
     plot ( sort ( genoSuccessInd ) , xlab = "individual genotyping success")
     plot ( sort ( genoSuccessLoci ), xlab = "locus genotyping success" )

}
#~~~ MODULE GENOTYPING ENDS~~~#

#~~~ MODULE SEXING ~~~#
{
     # males tat has low genotying sucess may also have low to zero coverage. Therefore they should be excldued.
     # we excldued individuals that has lower than 80% genotyoing sucess to exclude incorrect calling due to low genotyoing sucess. (therodhold can be change later based on expereience)
     # we can set two tehshold, such as 0.1 and 0.3 and leave in between unassigned. These therhoplds are arbitararyly selected.
     
     aveCOV1 = round (sapply ( 1:dim(COVS4)[2], function(i) { sum(unlist(COVS1[,i]),na.rm=T) / dim(COVS4)[1] }) )
     aveSDY = (unlist ( COVS4 [ SET_FINAL[,2] == "SDY_ion2" , ] ) / aveCOV1) # individual SDY coverage normlized to mean individual coverage
     highGenoSucInd = which(genoSuccessInd>0.80) # only individuals with genoSuccessInd>0.90 is selected.
     plot(aveSDY[highGenoSucInd],pch=20) # CLEAR BIMODALITY OBSERVED!!
     abline(h=0.3) 
     abline(h=0.1) 
 
}     
#~~~ MODULE SEXING ENDS~~~#  


